var map;					//! マップ オブジェクト。
var geo;					//! Geo コード取得用オブジェクト。
var isInitialized = false;	//!
var marker        = null;	//! マーカー。

/**
 * マップの初期化を行います。
 */
function initialize()
{
	var mapdiv    = document.getElementById( "map_canvas" );
	var myOptions = { zoom: 16, center: new google.maps.LatLng( 35.6894876, 139.6917064 ), mapTypeId: google.maps.MapTypeId.ROADMAP, scaleControl: true };
	map           = new google.maps.Map( mapdiv, myOptions );
	geo           = new google.maps.Geocoder();

	// マップの中央位置が更新された時のイベント
	google.maps.event.addListener( map, 'center_changed', function()
	{
		setTimeout( notifyLocation, 200 );
	});

	// マップのタイル読み込みが完了した時のイベント
	google.maps.event.addListener( map, 'tilesloaded', function()
	{
		// 起動時に一度だけ座標を更新する
		if( !isInitialized )
		{
			isInitialized = true;
			notifyLocation();
		}
	});
}

/**
 * マップにマーカーを追加します。
 * 既にマーカーが存在知る場合は何もしません。
 * マーカーの初期位置は、追加時点のマップ中央の座標となります。
 */
function addMarker()
{
	window.external.IsExistMarker = true;
	if( marker != null ) { return; }

	var center = map.getCenter();
	marker = new google.maps.Marker( { position: center, map: map, title: "印", draggable: true });
	notifyMarkerLocation( center );

	// マップの中央位置が更新された時のイベント
	google.maps.event.addListener( marker, 'drag', function()
	{
		notifyMarkerLocation( marker.position );
	});
}

/**
 * マップからマーカーを取り除きます。
 * マーカーが存在しない場合は何もしません。
 */
function removeMarker()
{
	window.external.IsExistMarker = false;
	if( marker == null ) { return; }

	marker.setMap();
	marker = null;
	notifyMarkerLocation( null );
}

/**
 * マーカーの緯度・経度をマネージコードへ通知します。
 */
function notifyMarkerLocation( latLng )
{
	window.external.MarkerLocation = ( latLng == null ? "" : "緯度 = " + latLng.lat() + "、経度 = " + latLng.lng() );
}

/**
 * 現在位置の緯度・経度を取得してマネージ コードへ通知します。
 */
function notifyLocation()
{
	var latLng = map.getCenter();
	window.external.Location = "緯度 = " + latLng.lat() + "、経度 = " + latLng.lng();
}

/**
 * 指定された住所の座標へ、マップを移動します。
 *
 * @param[in]	address	住所。
 */
function moveMap( address )
{
	if( geo )
	{
		geo.geocode( { 'address': address }, function( results, status )
		{
			if( results && results[ 0 ] )
			{
				map.setCenter( results[ 0 ].geometry.location );
				notifyLocation();
			}
		});
	}
}

/**
 * マーカーの座標へマップを移動します。
 */
function moveMapToMarker()
{
	if( marker == null ) { return; }

	map.setCenter( marker.position );
	notifyLocation();
}
